import * as grpc from "@grpc/grpc-js";
import { TopologyRpcService } from "../proto/rpc_grpc_pb.js";
export function init(node) {
    function subscribeCro(call, callback) {
        let returnCode = 0;
        try {
            node.subscribeObject(call.request.croId);
        }
        catch (e) {
            console.error(e);
            returnCode = 1;
        }
        const response = {
            returnCode,
        };
        callback(null, response);
    }
    function unsubscribeCro(call, callback) {
        let returnCode = 0;
        try {
            node.unsubscribeObject(call.request.croId);
        }
        catch (e) {
            console.error(e);
            returnCode = 1;
        }
        const response = {
            returnCode,
        };
        callback(null, response);
    }
    function getCroHashGraph(call, callback) {
        const hashes = [];
        try {
            const object = node.objectStore.get(call.request.croId);
            if (!object)
                throw Error("cro not found");
            for (const v of object.hashGraph.getAllVertices()) {
                hashes.push(v.hash);
            }
        }
        catch (e) {
            console.error(e);
        }
        const response = {
            verticesHashes: hashes,
        };
        callback(null, response);
    }
    const server = new grpc.Server();
    server.addService(TopologyRpcService, {
        subscribeCro,
        unsubscribeCro,
        getCroHashGraph,
    });
    server.bindAsync("0.0.0.0:6969", grpc.ServerCredentials.createInsecure(), (_error, _port) => {
        console.log("running grpc in port:", _port);
    });
}
